@@ -1,22 +1,23 @@
 /*
- *  Copyright 2001-2014 Adrian Thurston <thurston@complang.org>
- */
-
-/*  This file is part of Ragel.
+ * Copyright 2001-2014 Adrian Thurston <thurston@colm.net>
  *
- *  Ragel is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- * 
- *  Ragel is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- * 
- *  You should have received a copy of the GNU General Public License
- *  along with Ragel; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "ragel.h"
@@ -944,15 +945,26 @@ void Binary::LOCATE_COND()
 
 void Binary::GOTO( ostream &ret, int gotoDest, bool inFinish )
 {
-	ret << OPEN_GEN_BLOCK() << vCS() << " = " << gotoDest <<	
-		"; goto _again;" << CLOSE_GEN_BLOCK();
+	ret << OPEN_GEN_BLOCK() << vCS() << " = " << gotoDest <<	";";
+
+	if ( inFinish && !noEnd )
+		EOF_CHECK( ret );
+
+	ret << " goto _again;" << CLOSE_GEN_BLOCK();
 }
 
 void Binary::GOTO_EXPR( ostream &ret, GenInlineItem *ilItem, bool inFinish )
 {
 	ret << OPEN_GEN_BLOCK() << vCS() << " = " << OPEN_HOST_EXPR();
 	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
-	ret << CLOSE_HOST_EXPR() << "; goto _again;" << CLOSE_GEN_BLOCK();
+	ret << CLOSE_HOST_EXPR() << ";";
+
+	if ( inFinish && !noEnd )
+		EOF_CHECK( ret );
+	
+	ret << " goto _again;";
+	
+	ret << CLOSE_GEN_BLOCK();
 }
 
 void Binary::CURS( ostream &ret, bool inFinish )
@@ -989,7 +1001,14 @@ void Binary::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 
 	ret << STACK() << "[" << TOP() << "] = " <<
 			vCS() << "; " << TOP() << " += 1;" << vCS() << " = " << 
-			callDest << "; " << "goto _again;" << CLOSE_GEN_BLOCK();
+			callDest << ";";
+
+	if ( inFinish && !noEnd )
+		EOF_CHECK( ret );
+
+	ret << " goto _again;";
+
+	ret << CLOSE_GEN_BLOCK();
 }
 
 void Binary::CALL_EXPR( ostream &ret, GenInlineItem *ilItem, int targState, bool inFinish )
@@ -1006,7 +1025,14 @@ void Binary::CALL_EXPR( ostream &ret, GenInlineItem *ilItem, int targState, bool
 			vCS() << "; " << TOP() << " += 1;" << vCS() <<
 			" = " << OPEN_HOST_EXPR();
 	INLINE_LIST( ret, ilItem->children, targState, inFinish, false );
-	ret << CLOSE_HOST_EXPR() << "; " << "goto _again;" << CLOSE_GEN_BLOCK();
+	ret << CLOSE_HOST_EXPR() << ";";
+
+	if ( inFinish && !noEnd )
+		EOF_CHECK( ret );
+
+	ret << " goto _again;";
+
+	ret << CLOSE_GEN_BLOCK();
 }
 
 void Binary::NCALL( ostream &ret, int callDest, int targState, bool inFinish )
@@ -1051,6 +1077,9 @@ void Binary::RET( ostream &ret, bool inFinish )
 		ret << CLOSE_HOST_BLOCK();
 	}
 
+	if ( inFinish && !noEnd )
+		EOF_CHECK( ret );
+
 	ret << "goto _again;" << CLOSE_GEN_BLOCK();
 }
 
