@@ -1,22 +1,23 @@
 /*
- *  Copyright 2001-2014 Adrian Thurston <thurston@complang.org>
- */
-
-/*  This file is part of Ragel.
+ * Copyright 2001-2014 Adrian Thurston <thurston@colm.net>
  *
- *  Ragel is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- * 
- *  Ragel is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- * 
- *  You should have received a copy of the GNU General Public License
- *  along with Ragel; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "ragel.h"
@@ -73,21 +74,46 @@ void IpGoto::genAnalysis()
 
 bool IpGoto::useAgainLabel()
 {
-	return redFsm->anyRegActionRets() || 
-			redFsm->anyRegActionByValControl() || 
+	return redFsm->anyActionRets() || 
+			redFsm->anyActionByValControl() ||
 			redFsm->anyRegNextStmt();
 }
 
+void IpGoto::EOF_CHECK( ostream &ret, int gotoDest )
+{
+	ret << 
+		"       if ( " << P() << " == " << PE() << " )\n"
+		"               goto _test_eof" << gotoDest << ";\n";
+
+	testEofUsed = true;
+}
+
 void IpGoto::GOTO( ostream &ret, int gotoDest, bool inFinish )
 {
-	ret << OPEN_GEN_BLOCK() << "goto st" << gotoDest << ";" << CLOSE_GEN_BLOCK();
+	ret << OPEN_GEN_BLOCK();
+
+	if ( inFinish && !noEnd )
+		EOF_CHECK( ret, gotoDest );
+
+	ret << "goto st" << gotoDest << ";";
+
+	ret << CLOSE_GEN_BLOCK();
 }
 
 void IpGoto::GOTO_EXPR( ostream &ret, GenInlineItem *ilItem, bool inFinish )
 {
 	ret << OPEN_GEN_BLOCK() << vCS() << " = " << OPEN_HOST_EXPR();
 	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
-	ret << CLOSE_HOST_EXPR() << "; " << "goto _again;" << CLOSE_GEN_BLOCK();
+	ret << CLOSE_HOST_EXPR() << ";";
+
+	/* Since we are setting CS above and can select on it, call the all-state
+	 * test_eof. */
+	if ( inFinish && !noEnd )
+		CodeGen::EOF_CHECK( ret );
+	
+	ret << " goto _again;";
+	
+	ret << CLOSE_GEN_BLOCK();
 }
 
 void IpGoto::CALL( ostream &ret, int callDest, int targState, bool inFinish )
@@ -101,8 +127,14 @@ void IpGoto::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 	}
 
 	ret << STACK() << "[" << TOP() << "] = " << targState << 
-			"; " << TOP() << "+= 1; " << "goto st" << callDest << ";" <<
-			CLOSE_GEN_BLOCK();
+			"; " << TOP() << "+= 1; ";
+
+	if ( inFinish && !noEnd )
+		EOF_CHECK( ret, callDest );
+
+	ret << "goto st" << callDest << ";";
+
+	ret << CLOSE_GEN_BLOCK();
 }
 
 void IpGoto::NCALL( ostream &ret, int callDest, int targState, bool inFinish )
@@ -133,7 +165,16 @@ void IpGoto::CALL_EXPR( ostream &ret, GenInlineItem *ilItem, int targState, bool
 	ret << STACK() << "[" << TOP() << "] = " << targState << "; " << TOP() << "+= 1;" <<
 			vCS() << " = " << OPEN_HOST_EXPR();
 	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
-	ret << CLOSE_HOST_EXPR() << "; goto _again;" << CLOSE_GEN_BLOCK();
+	ret << CLOSE_HOST_EXPR() << ";";
+
+	/* Since we are setting CS above and can select on it, call the all-state
+	 * test_eof. */
+	if ( inFinish && !noEnd )
+		CodeGen::EOF_CHECK( ret );
+
+	ret << " goto _again;";
+	
+	ret << CLOSE_GEN_BLOCK();
 }
 
 void IpGoto::NCALL_EXPR( ostream &ret, GenInlineItem *ilItem, int targState, bool inFinish )
@@ -163,6 +204,9 @@ void IpGoto::RET( ostream &ret, bool inFinish )
 		ret << CLOSE_HOST_BLOCK();
 	}
 
+	if ( inFinish && !noEnd )
+		CodeGen::EOF_CHECK( ret );
+
 	ret << "goto _again;" << CLOSE_GEN_BLOCK();
 }
 
@@ -609,8 +653,8 @@ void IpGoto::setLabelsNeeded( RedCondPair *pair )
 /* Set up labelNeeded flag for each state. */
 void IpGoto::setLabelsNeeded()
 {
-	/* If we use the _again label, then we the _again switch, which uses all
-	 * labels. */
+	/* If we use the _again label, then we generate the _again switch, which
+	 * uses all labels. */
 	if ( useAgainLabel() ) {
 		for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ )
 			st->labelNeeded = true;
@@ -627,6 +671,13 @@ void IpGoto::setLabelsNeeded()
 
 		for ( CondApSet::Iter cond = redFsm->condSet; cond.lte(); cond++ )
 			setLabelsNeeded( &cond->p );
+
+		for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+			if ( st->eofAction != 0 ) {
+				for ( GenActionTable::Iter item = st->eofAction->key; item.lte(); item++ )
+					setLabelsNeeded( item->value->inlineList );
+			}
+		}
 	}
 
 	if ( !noEnd ) {
@@ -634,6 +685,7 @@ void IpGoto::setLabelsNeeded()
 			if ( st != redFsm->errState )
 				st->outNeeded = st->labelNeeded;
 		}
+
 	}
 }
 
